@@ -99,6 +99,8 @@ public class Operator extends NativeObject {
         }
     }
 
+    public final OperatorInfo info;
+
     /**
      * Construct an OpenDAL operator:
      *
@@ -109,8 +111,21 @@ public class Operator extends NativeObject {
      * @param schema the name of the underneath service to access data from.
      * @param map    a map of properties to construct the underneath operator.
      */
-    public Operator(String schema, Map<String, String> map) {
-        super(constructor(schema, map));
+    public static Operator of(String schema, Map<String, String> map) {
+        final long nativeHandle = constructor(schema, map);
+        final OperatorInfo info = makeOperatorInfo(nativeHandle);
+        return new Operator(nativeHandle, info);
+    }
+
+    Operator(long nativeHandle, OperatorInfo info) {
+        super(nativeHandle);
+        this.info = info;
+    }
+
+    public BlockingOperator blocking() {
+        final long nativeHandle = makeBlockingOp(this.nativeHandle);
+        final OperatorInfo info = this.info;
+        return new BlockingOperator(nativeHandle, info);
     }
 
     public CompletableFuture<Void> write(String path, String content) {
@@ -137,7 +152,7 @@ public class Operator extends NativeObject {
         return f.thenApply(Metadata::new);
     }
 
-    public CompletableFuture<String> read(String path) {
+    public CompletableFuture<byte[]> read(String path) {
         final long requestId = read(nativeHandle, path);
         return AsyncRegistry.take(requestId);
     }
@@ -162,6 +177,16 @@ public class Operator extends NativeObject {
         return AsyncRegistry.take(requestId);
     }
 
+    public CompletableFuture<Void> createDir(String path) {
+        final long requestId = createDir(nativeHandle, path);
+        return AsyncRegistry.take(requestId);
+    }
+
+    public CompletableFuture<Void> copy(String sourcePath, String targetPath) {
+        final long requestId = copy(nativeHandle, sourcePath, targetPath);
+        return AsyncRegistry.take(requestId);
+    }
+
     @Override
     protected native void disposeInternal(long handle);
 
@@ -182,4 +207,12 @@ public class Operator extends NativeObject {
     private static native long presignWrite(long nativeHandle, String path, long duration);
 
     private static native long presignStat(long nativeHandle, String path, long duration);
+
+    private static native OperatorInfo makeOperatorInfo(long nativeHandle);
+
+    private static native long makeBlockingOp(long nativeHandle);
+
+    private static native long createDir(long nativeHandle, String path);
+
+    private static native long copy(long nativeHandle, String sourcePath, String targetPath);
 }
