@@ -224,7 +224,11 @@ impl Operator {
             Either::B(s) => s.into_bytes(),
         };
 
-        self.0.append(&path, c).await.map_err(format_napi_error)
+        self.0
+            .write_with(&path, c)
+            .append(true)
+            .await
+            .map_err(format_napi_error)
     }
 
     /// Copy file according to given `from` and `to` path.
@@ -288,6 +292,7 @@ impl Operator {
     /// An error will be returned if given path doesn't end with /.
     ///
     /// ### Example
+    ///
     /// ```javascript
     /// const lister = await op.scan("/path/to/dir/");
     /// while (true) {
@@ -303,7 +308,13 @@ impl Operator {
     /// `````
     #[napi]
     pub async fn scan(&self, path: String) -> Result<Lister> {
-        Ok(Lister(self.0.scan(&path).await.map_err(format_napi_error)?))
+        Ok(Lister(
+            self.0
+                .lister_with(&path)
+                .delimiter("")
+                .await
+                .map_err(format_napi_error)?,
+        ))
     }
 
     /// List dir in flat way synchronously.
@@ -329,7 +340,12 @@ impl Operator {
     #[napi]
     pub fn scan_sync(&self, path: String) -> Result<BlockingLister> {
         Ok(BlockingLister(
-            self.0.blocking().scan(&path).map_err(format_napi_error)?,
+            self.0
+                .blocking()
+                .lister_with(&path)
+                .delimiter("")
+                .call()
+                .map_err(format_napi_error)?,
         ))
     }
 
@@ -408,7 +424,9 @@ impl Operator {
     /// ```
     #[napi]
     pub async fn list(&self, path: String) -> Result<Lister> {
-        Ok(Lister(self.0.list(&path).await.map_err(format_napi_error)?))
+        Ok(Lister(
+            self.0.lister(&path).await.map_err(format_napi_error)?,
+        ))
     }
 
     /// List given path synchronously.
@@ -434,7 +452,12 @@ impl Operator {
     #[napi]
     pub fn list_sync(&self, path: String) -> Result<BlockingLister> {
         Ok(BlockingLister(
-            self.0.blocking().scan(&path).map_err(format_napi_error)?,
+            self.0
+                .blocking()
+                .lister_with(&path)
+                .delimiter("")
+                .call()
+                .map_err(format_napi_error)?,
         ))
     }
 
