@@ -28,9 +28,6 @@ pub use read::*;
 mod write;
 pub use write::*;
 
-mod append;
-pub use append::*;
-
 mod stream;
 pub use stream::*;
 
@@ -39,7 +36,9 @@ pub use page::*;
 
 mod cursor;
 pub use cursor::Cursor;
-pub use cursor::VectorCursor;
 
 mod entry;
 pub use entry::Entry;
+
+mod buf;
+pub use buf::*;
