@@ -88,6 +88,10 @@ pub enum Scheme {
     Redis,
     /// [postgresql][crate::services::Postgresql]: Postgresql services
     Postgresql,
+    /// [mysql][crate::services::Mysql]: Mysql services
+    Mysql,
+    /// [sqlite][crate::services::Sqlite]: Sqlite services
+    Sqlite,
     /// [rocksdb][crate::services::Rocksdb]: RocksDB services
     Rocksdb,
     /// [s3][crate::services::S3]: AWS S3 alike services.
@@ -167,6 +171,8 @@ impl FromStr for Scheme {
             "ipmfs" => Ok(Scheme::Ipmfs),
             "memcached" => Ok(Scheme::Memcached),
             "memory" => Ok(Scheme::Memory),
+            "mysql" => Ok(Scheme::Mysql),
+            "sqlite" => Ok(Scheme::Sqlite),
             "mini_moka" => Ok(Scheme::MiniMoka),
             "moka" => Ok(Scheme::Moka),
             "obs" => Ok(Scheme::Obs),
@@ -218,6 +224,7 @@ impl From<Scheme> for &'static str {
             Scheme::Onedrive => "onedrive",
             Scheme::Persy => "persy",
             Scheme::Postgresql => "postgresql",
+            Scheme::Mysql => "mysql",
             Scheme::Gdrive => "gdrive",
             Scheme::Dropbox => "dropbox",
             Scheme::Redis => "redis",
@@ -233,6 +240,7 @@ impl From<Scheme> for &'static str {
             Scheme::Webhdfs => "webhdfs",
             Scheme::Redb => "redb",
             Scheme::Tikv => "tikv",
+            Scheme::Sqlite => "sqlite",
             Scheme::Custom(v) => v,
         }
     }
