@@ -20,7 +20,7 @@ use anyhow::Result;
 use crate::*;
 
 pub fn behavior_copy_tests(op: &Operator) -> Vec<Trial> {
-    let cap = op.info().capability();
+    let cap = op.info().full_capability();
 
     if !(cap.read && cap.write && cap.copy) {
         return vec![];
@@ -28,7 +28,8 @@ pub fn behavior_copy_tests(op: &Operator) -> Vec<Trial> {
 
     async_trials!(
         op,
-        test_copy_file,
+        test_copy_file_with_ascii_name,
+        test_copy_file_with_non_ascii_name,
         test_copy_non_existing_source,
         test_copy_source_dir,
         test_copy_target_dir,
@@ -38,8 +39,8 @@ pub fn behavior_copy_tests(op: &Operator) -> Vec<Trial> {
     )
 }
 
-/// Copy a file and test with stat.
-pub async fn test_copy_file(op: Operator) -> Result<()> {
+/// Copy a file with ascii name and test contents.
+pub async fn test_copy_file_with_ascii_name(op: Operator) -> Result<()> {
     let source_path = uuid::Uuid::new_v4().to_string();
     let (source_content, _) = gen_bytes();
 
@@ -57,6 +58,23 @@ pub async fn test_copy_file(op: Operator) -> Result<()> {
     Ok(())
 }
 
+/// Copy a file with non ascii name and test contents.
+pub async fn test_copy_file_with_non_ascii_name(op: Operator) -> Result<()> {
+    let source_path = "🐂🍺中文.docx";
+    let target_path = "😈🐅Français.docx";
+    let (source_content, _) = gen_bytes();
+
+    op.write(source_path, source_content.clone()).await?;
+    op.copy(source_path, target_path).await?;
+
+    let target_content = op.read(target_path).await.expect("read must succeed");
+    assert_eq!(target_content, source_content);
+
+    op.delete(source_path).await.expect("delete must succeed");
+    op.delete(target_path).await.expect("delete must succeed");
+    Ok(())
+}
+
 /// Copy a nonexistent source should return an error.
 pub async fn test_copy_non_existing_source(op: Operator) -> Result<()> {
     let source_path = uuid::Uuid::new_v4().to_string();
